package com.sintecmedia.components.component;

import javax.el.MethodExpression;
import javax.faces.component.ActionSource2;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.FacesEvent;

import org.ajax4jsf.component.AjaxActionComponent;
import org.apache.log4j.Logger;

import com.sintecmedia.components.event.DaySelectedEvent;
import com.sintecmedia.components.event.DaySelectedListener;
import com.sintecmedia.components.exceptions.DayIndexOutOfBoundsException;

public abstract class UIWeekDaysButtons 
		extends AjaxActionComponent implements ActionSource2 {
	
	private static final Logger LOG = 
		Logger.getLogger( UIWeekDaysButtons.class );
	public static final String COMPONENT_TYPE = 
		"com.sintecmedia.components.WeekDaysButtons";
	public static final String COMPONENT_FAMILY = 
		"com.sintecmedia.components.WeekDaysButtons";	

	public static final int[] DAY_MASKS = { 1, 2, 4, 8, 16, 32, 64 };
	
	/**
	 * 
	 * @param index index of day starting from Sunday
	 * @return true if day with specified index is selected
	 */
	public boolean isDaySelected( int index ) {
		int intVal = 
			getValue() != null ? Integer.parseInt( getValue().toString() ) : 0;
		return ( intVal & UIWeekDaysButtons.DAY_MASKS[ index] ) > 0;		
	}
	
	/**
	 * Changes day state: selects it if it was unselected, else unselects it
	 * @param index index of day to change, starting from Sunday
	 */
	public void flipDay( int index ) {		
		if( isDaySelected( index ) ) {
			unselectDay( index );
		} else {
			selectDay( index );
		}
	}
	
	/**
	 * Selects day with index specified.
	 * @param index index of day to select, starting from Sunday
	 */
	public void selectDay( int index ) {		
		int intVal = 
			getValue() != null ? Integer.parseInt( getValue().toString() ) : 0;		
		setValue( intVal | DAY_MASKS[ index ] );
	}

	/**
	 * Unselects day with index specified.
	 * @param index index of day to unselect, starting from Sunday
	 */
	public void unselectDay( int index ) {
		int intVal = 
			getValue() != null ? Integer.parseInt( getValue().toString() ) : 0;
		setValue( intVal & ~DAY_MASKS[ index ] );
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponentBase#broadcast(javax.faces.event.FacesEvent)
	 */
	public void broadcast( FacesEvent event ) 
			throws AbortProcessingException, DayIndexOutOfBoundsException {
		super.broadcast( event );
		if( !( event instanceof DaySelectedEvent ) ) {
			return;
		}		
		LOG.debug( "broadcast(): source val: " + getValue() );
		
		//Check parameters.
		DaySelectedEvent daySelectedEvent = (DaySelectedEvent) event;
		int index = daySelectedEvent.getDayIndex();
		if( index < 0 && index >= DAY_MASKS.length ) {
			DayIndexOutOfBoundsException ex =  
				new DayIndexOutOfBoundsException();
			ex.setComponent( this );
			ex.setMethodName( "broadcast" );
			throw ex;
		}
		
		//Update selected days mask		
		flipDay( index );
		
		//Invoke listener
		MethodExpression methodExpression = getDaySelectedListener();
		if( methodExpression != null ) {
			FacesContext facesContext = getFacesContext();
		    methodExpression.invoke( 
		   		facesContext.getELContext(), new Object[] { event } );
		}				
	}	

	public DaySelectedListener[] getDaySelectedListeners() {
		return (DaySelectedListener[]) getFacesListeners(
				DaySelectedListener.class);
	}
		
	public void addDaySelectedListener( DaySelectedListener listener ) {
		addFacesListener(listener);
	}
	
	public void removeDaySelectedListener( DaySelectedListener listener ) {
        removeFacesListener(listener);
	}
		
	public abstract MethodExpression getDaySelectedListener();

	public abstract void setDaySelectedListener( MethodExpression scrollerListener );
}
